package com.gic.operation.web.controller;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.constant.UserConstants;
import com.gic.auth.dto.UserDTO;
import com.gic.auth.service.UserApiService;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.constant.EnterpriseListOrderByEnum;
import com.gic.enterprise.constant.EnterpriseVersionStatusEnum;
import com.gic.enterprise.dto.EnterpriseDTO;
import com.gic.enterprise.dto.EnterpriseInitDTO;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.response.EnterpriseRestResponse;
import com.gic.enterprise.service.EnterpriseApiService;
import com.gic.enterprise.service.EnterpriseInitApiService;
import com.gic.operation.web.qo.PageQO;
import com.gic.operation.web.utils.UserPasswordUtil;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author guojx
 * @date 2019/7/17 10:32 AM
 */
@RestController
@RequestMapping("/enterprise")
public class EnterpriseController {
    private static final Logger LOGGER = LoggerFactory.getLogger(EnterpriseController.class);
    @Autowired
    private EnterpriseApiService enterpriseApiService;
    @Autowired
    private UserApiService userApiService;
    @Autowired
    private EnterpriseInitApiService enterpriseInitApiService;

    @RequestMapping("/save-enterprise")
    public RestResponse saveEnterprise(@Validated({EnterpriseDTO.SaveEnterpriseValid.class}) EnterpriseDTO enterpriseDTO,
                                       @Validated({UserDTO.UserQoValid.class}) UserDTO userDTO) {
        int passwordType = userDTO.getPasswordType();
        String password;
        //加密
        if (UserConstants.CREATE_AUTO == passwordType) {
            //自动随机生成
            password = UserPasswordUtil.createPasswordAuto();
        } else {
            if (StringUtils.isNotBlank(userDTO.getPassword()) && userDTO.getPassword().equals(userDTO.getConfirmPassword())) {
                //如果一致
                password = userDTO.getPassword();
            } else {
                return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "密码不一致");
            }
        }
        userDTO.setPassword(UserPasswordUtil.getEncryptPassword(password));
        ServiceResponse<Integer> enterpriseResult = enterpriseApiService.saveEnterprise(enterpriseDTO, userDTO);
        if (enterpriseResult.isSuccess()) {
           return RestResponse.success();
        } else {
            return EnterpriseRestResponse.failure(enterpriseResult);
        }
    }

    @RequestMapping("/list-enterprise")
    public RestResponse listEnterprise(String search, Integer serviceVersion, Integer serviceStatus, Integer orderBy, PageQO pageQO) {
        if (orderBy == null) {
            orderBy = 0;
        }
        ServiceResponse enterpriseResult = enterpriseApiService.listEnterprise(search, serviceVersion, serviceStatus,
                EnterpriseListOrderByEnum.getOrderBy(orderBy), pageQO.getCurrentPage(), pageQO.getPageSize());
        if (enterpriseResult.isSuccess()) {
            return RestResponse.success(enterpriseResult.getResult());
        } else {
            return EnterpriseRestResponse.failure(enterpriseResult);
        }
    }

    @RequestMapping("/list-enterprise-version-status")
    public RestResponse listEnterpriseVersionStatus() {
        Map<Integer, String> resultMap = new HashMap<>(16);
        for (EnterpriseVersionStatusEnum statusEnum : EnterpriseVersionStatusEnum.values()) {
            resultMap.put(statusEnum.getCode(), statusEnum.getMessage());
        }
        return RestResponse.success(resultMap);
    }

    @RequestMapping("/init-enterprise")
    public RestResponse initEnterprise(Integer enterpriseId) {
        ServiceResponse enterpriseResult = enterpriseApiService.initEnterprise(enterpriseId);
        if (enterpriseResult.isSuccess()) {
            return RestResponse.success();
        } else {
            return EnterpriseRestResponse.failure(enterpriseResult);
        }
    }

    @RequestMapping("/list-enterprise-init-info")
    public RestResponse listEnterpriseInitInfo(Integer enterpriseId) {
        ServiceResponse<List<EnterpriseInitDTO>> response = enterpriseInitApiService.listByEnterpriseId(enterpriseId);
        if (response.isSuccess()) {
            List<EnterpriseInitDTO> dtoList = response.getResult();
            return RestResponse.success(changeListToTree(0, dtoList));
        } else {
            return EnterpriseRestResponse.failure(response);
        }
    }


    @RequestMapping("/edit-enterprise")
    public RestResponse editEnterprise(@Validated({EnterpriseDTO.EditEnterpriseValid.class, EnterpriseDTO.SaveEnterpriseValid.class})
            EnterpriseDTO enterpriseDTO) {
        ServiceResponse enterpriseResult = enterpriseApiService.editEnterprise(enterpriseDTO);
        if (enterpriseResult.isSuccess()) {
            return RestResponse.success();
        } else {
            return EnterpriseRestResponse.failure(enterpriseResult);
        }
    }

    @RequestMapping("/get-enterprise")
    public RestResponse editEnterprise(Integer enterpriseId) {
        ServiceResponse<EnterpriseDTO> enterpriseResult = enterpriseApiService.getEnterpriseById(enterpriseId);
        if (enterpriseResult.isSuccess()) {
            return RestResponse.success(enterpriseResult.getResult());
        } else {
            return EnterpriseRestResponse.failure(enterpriseResult);
        }
    }

    @RequestMapping("/enable-enterprise")
    public RestResponse enableEnterprise(Integer enterpriseId) {
        ServiceResponse enterpriseResult = enterpriseApiService.enableEnterprise(enterpriseId);
        if (enterpriseResult.isSuccess()) {
            return RestResponse.success();
        } else {
            return EnterpriseRestResponse.failure(enterpriseResult);
        }
    }

    @RequestMapping("/disable-enterprise")
    public RestResponse disableEnterprise(Integer enterpriseId) {
        ServiceResponse enterpriseResult = enterpriseApiService.disableEnterprise(enterpriseId);
        if (enterpriseResult.isSuccess()) {
            return RestResponse.success();
        } else {
            return EnterpriseRestResponse.failure(enterpriseResult);
        }
    }

    public static List<Map<String, Object>> changeListToTree(int initId, List<EnterpriseInitDTO> list) {
        List<Map<String, Object>> result = new ArrayList<>();
        if (list != null) {
            for (EnterpriseInitDTO t : list) {
                if (initId == t.getParentId()) {
                    Map<String, Object> data = new HashMap<>(16);
                    data.put("parentId", t.getParentId());
                    data.put("initId", t.getEnterpriseInitId());
                    data.put("configName", t.getConfigName());
                    data.put("sort", t.getSort());
                    data.put("isInitConfig", t.getIsInitConfig());
                    data.put("level", t.getLevel());
                    data.put("configUrl", t.getConfigUrl());
                    List<Map<String, Object>> children = changeListToTree(t.getEnterpriseInitId(), list);
                    data.put("children", children);
                    result.add(data);
                }
            }
        }
        return result;
    }
}
